"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useUnderlayParams = useUnderlayParams;
exports.useOverlayParams = useOverlayParams;
exports.useSwipeableItemParams = useSwipeableItemParams;
exports.default = exports.OpenDirection = void 0;

var _react = _interopRequireWildcard(require("react"));

var _reactNative = require("react-native");

var _reactNativeGestureHandler = require("react-native-gesture-handler");

var _reactNativeReanimated = _interopRequireWildcard(require("react-native-reanimated"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

let OpenDirection;
exports.OpenDirection = OpenDirection;

(function (OpenDirection) {
  OpenDirection["LEFT"] = "left";
  OpenDirection["RIGHT"] = "right";
  OpenDirection["NONE"] = "none";
})(OpenDirection || (exports.OpenDirection = OpenDirection = {}));

const renderNull = () => null;

const UnderlayContext = /*#__PURE__*/_react.default.createContext(undefined);

const OverlayContext = /*#__PURE__*/_react.default.createContext(undefined);

function SwipeableItem(props, ref) {
  const {
    item,
    children,
    renderOverlay = renderNull,
    renderUnderlayLeft = renderNull,
    renderUnderlayRight = renderNull,
    snapPointsLeft = [],
    snapPointsRight = [],
    swipeEnabled,
    activationThreshold = 20,
    overSwipe = 20,
    swipeDamping = 10,
    onChange = () => {},
    animationConfig = {}
  } = props;
  const springConfig = {
    damping: 20,
    mass: 0.2,
    stiffness: 100,
    overshootClamping: false,
    restSpeedThreshold: 0.5,
    restDisplacementThreshold: 0.5,
    ...animationConfig
  };
  const [openDirection, setOpenDirection] = (0, _react.useState)(OpenDirection.NONE);
  const animStatePos = (0, _reactNativeReanimated.useSharedValue)(0);
  const isGestureActive = (0, _reactNativeReanimated.useSharedValue)(false);
  const swipingLeft = (0, _reactNativeReanimated.useDerivedValue)(() => animStatePos.value < 0, [animStatePos]);
  const swipingRight = (0, _reactNativeReanimated.useDerivedValue)(() => animStatePos.value > 0, [animStatePos]);
  const maxSnapPointLeft = -1 * (Math.max(...snapPointsLeft) || 0);
  const maxSnapPointRight = Math.max(...snapPointsRight) || 0;
  const maxTranslateLeft = maxSnapPointLeft - overSwipe;
  const maxTranslateRight = maxSnapPointRight + overSwipe;
  const percentOpenLeft = (0, _reactNativeReanimated.useDerivedValue)(() => {
    return swipingLeft.value ? Math.abs(animStatePos.value / maxSnapPointLeft) : 0;
  }, []);
  const percentOpenRight = (0, _reactNativeReanimated.useDerivedValue)(() => {
    return swipingRight.value ? Math.abs(animStatePos.value / maxSnapPointRight) : 0;
  }, []);
  const hasLeft = !!(snapPointsLeft !== null && snapPointsLeft !== void 0 && snapPointsLeft.length);
  const hasRight = !!(snapPointsRight !== null && snapPointsRight !== void 0 && snapPointsRight.length);
  const activeOffsetL = hasLeft || openDirection === OpenDirection.RIGHT ? -activationThreshold : -Number.MAX_VALUE;
  const activeOffsetR = hasRight || openDirection === OpenDirection.LEFT ? activationThreshold : Number.MAX_VALUE;
  const activeOffsetX = [activeOffsetL, activeOffsetR];
  const leftStyle = (0, _reactNativeReanimated.useAnimatedStyle)(() => ({
    opacity: percentOpenLeft.value > 0 ? 1 : 0
  }), []);
  const rightStyle = (0, _reactNativeReanimated.useAnimatedStyle)(() => ({
    opacity: percentOpenRight.value > 0 ? 1 : 0
  }), []);
  const overlayStyle = (0, _reactNativeReanimated.useAnimatedStyle)(() => ({
    transform: [{
      translateX: animStatePos.value
    }]
  }), [animStatePos]);

  function openLeft(snapPoint) {
    return new Promise(resolve => {
      function resolvePromiseIfFinished(isFinished) {
        if (isFinished) resolve();
      }

      animStatePos.value = (0, _reactNativeReanimated.withSpring)(snapPoint !== null && snapPoint !== void 0 ? snapPoint : maxSnapPointLeft, springConfig, isFinished => {
        (0, _reactNativeReanimated.runOnJS)(resolvePromiseIfFinished)(isFinished);
      });
    });
  }

  function openRight(snapPoint) {
    return new Promise(resolve => {
      function resolvePromiseIfFinished(isFinished) {
        if (isFinished) resolve();
      }

      animStatePos.value = (0, _reactNativeReanimated.withSpring)(snapPoint !== null && snapPoint !== void 0 ? snapPoint : maxSnapPointRight, springConfig, isFinished => {
        (0, _reactNativeReanimated.runOnJS)(resolvePromiseIfFinished)(isFinished);
      });
    });
  }

  function close() {
    return new Promise(resolve => {
      function resolvePromiseIfFinished(isFinished) {
        if (isFinished) resolve();
      }

      animStatePos.value = (0, _reactNativeReanimated.withSpring)(0, springConfig, isFinished => {
        (0, _reactNativeReanimated.runOnJS)(resolvePromiseIfFinished)(isFinished);
      });
    });
  }

  (0, _react.useImperativeHandle)(ref, () => ({
    open: (openDirection, snapPoint) => {
      if (openDirection === OpenDirection.LEFT) return openLeft(snapPoint);
      if (openDirection === OpenDirection.RIGHT) return openRight(snapPoint);
      return close();
    },
    close
  }));

  function onAnimationEnd(openDirection, snapPoint) {
    setOpenDirection(openDirection);
    onChange({
      open: openDirection,
      snapPoint
    });
  }

  const onGestureEvent = (0, _reactNativeReanimated.useAnimatedGestureHandler)({
    onStart: (evt, ctx) => {
      ctx.startX = animStatePos.value;
      isGestureActive.value = true;
    },
    onActive: (evt, ctx) => {
      // console.log("onActive!!", evt)
      const rawVal = evt.translationX + ctx.startX;
      const clampedVal = Math.min(Math.max(maxTranslateLeft, rawVal), maxTranslateRight);
      animStatePos.value = clampedVal;
    },
    onEnd: evt => {
      isGestureActive.value = false; // Approximate where item would end up with velocity taken into account

      const velocityModifiedPosition = animStatePos.value + evt.velocityX / swipeDamping;
      const allSnapPoints = snapPointsLeft.map(p => p * -1).concat(snapPointsRight); // The user is not required to designate [0] in their snap point array,
      // but we need to make sure 0 is a snap point.

      allSnapPoints.push(0);
      const closestSnapPoint = allSnapPoints.reduce((acc, cur) => {
        const diff = Math.abs(velocityModifiedPosition - cur);
        const prevDiff = Math.abs(velocityModifiedPosition - acc);
        return diff < prevDiff ? cur : acc;
      }, Infinity);

      const onComplete = () => {
        const openDirection = closestSnapPoint === 0 ? OpenDirection.NONE : closestSnapPoint > 0 ? OpenDirection.RIGHT : OpenDirection.LEFT;
        (0, _reactNativeReanimated.runOnJS)(onAnimationEnd)(openDirection, Math.abs(closestSnapPoint));
      };

      if (animStatePos.value === closestSnapPoint) onComplete();else animStatePos.value = (0, _reactNativeReanimated.withSpring)(closestSnapPoint, springConfig, onComplete);
    }
  });
  const sharedParams = (0, _react.useMemo)(() => ({
    item,
    isGestureActive,
    close
  }), []);
  const underlayRightParams = (0, _react.useMemo)(() => {
    return {
      open: openRight,
      percentOpen: percentOpenRight,
      direction: OpenDirection.RIGHT,
      ...sharedParams
    };
  }, [percentOpenRight, openRight, sharedParams]);
  const underlayLeftParams = (0, _react.useMemo)(() => {
    return {
      open: openLeft,
      percentOpen: percentOpenLeft,
      direction: OpenDirection.LEFT,
      ...sharedParams
    };
  }, [item, percentOpenLeft, openLeft, sharedParams]);
  const overlayParams = (0, _react.useMemo)(() => {
    // If there is only one swipe direction, use it as the 'open' function. Otherwise we need to choose one.
    const open = hasLeft && !hasRight ? openLeft : hasRight && !hasLeft ? openRight : openLeft;
    return {
      openLeft: openLeft,
      openRight: openRight,
      percentOpenLeft,
      percentOpenRight,
      openDirection,
      open,
      ...sharedParams
    };
  }, [openLeft, openRight, openDirection, percentOpenLeft, percentOpenRight, hasLeft, hasRight]);
  return /*#__PURE__*/_react.default.createElement(OverlayContext.Provider, {
    value: overlayParams
  }, /*#__PURE__*/_react.default.createElement(_reactNativeReanimated.default.View, {
    pointerEvents: openDirection === OpenDirection.LEFT ? "auto" : "none",
    style: [styles.underlay, leftStyle]
  }, /*#__PURE__*/_react.default.createElement(UnderlayContext.Provider, {
    value: underlayLeftParams
  }, renderUnderlayLeft(underlayLeftParams))), /*#__PURE__*/_react.default.createElement(_reactNativeReanimated.default.View, {
    pointerEvents: openDirection === OpenDirection.RIGHT ? "auto" : "none",
    style: [styles.underlay, rightStyle]
  }, /*#__PURE__*/_react.default.createElement(UnderlayContext.Provider, {
    value: underlayRightParams
  }, renderUnderlayRight(underlayRightParams))), /*#__PURE__*/_react.default.createElement(_reactNativeGestureHandler.PanGestureHandler, {
    enabled: swipeEnabled,
    activeOffsetX: activeOffsetX,
    onGestureEvent: onGestureEvent
  }, /*#__PURE__*/_react.default.createElement(_reactNativeReanimated.default.View, {
    style: [styles.flex, overlayStyle]
  }, children, renderOverlay(overlayParams))));
}

var _default = /*#__PURE__*/_react.default.forwardRef(SwipeableItem);

exports.default = _default;

function useUnderlayParams() {
  const underlayContext = (0, _react.useContext)(UnderlayContext);

  if (!underlayContext) {
    throw new Error("useUnderlayParams must be called from within an UnderlayContext.Provider!");
  }

  return underlayContext;
}

function useOverlayParams() {
  const overlayContext = (0, _react.useContext)(OverlayContext);

  if (!overlayContext) {
    throw new Error("useOverlayParams must be called from within an OverlayContext.Provider!");
  }

  return overlayContext;
}

function useSwipeableItemParams() {
  const overlayContext = (0, _react.useContext)(OverlayContext);

  if (!overlayContext) {
    throw new Error("useOverlayParams must be called from within an OverlayContext.Provider!");
  }

  const underlayContext = (0, _react.useContext)(UnderlayContext);
  const contextDirection = underlayContext === null || underlayContext === void 0 ? void 0 : underlayContext.direction;
  const open = (0, _react.useCallback)((snapPoint, direction) => {
    const openFnLeft = overlayContext.openLeft;
    const openFnRight = overlayContext.openRight;
    const openDirection = direction || contextDirection;
    const openFn = openDirection === OpenDirection.LEFT ? openFnLeft : openFnRight;
    return openFn(snapPoint);
  }, [overlayContext, contextDirection]);
  const percentOpen = (0, _react.useMemo)(() => {
    if (contextDirection) {
      // If we're calling from within an underlay context, return the open percentage of that underlay
      return contextDirection === OpenDirection.LEFT ? overlayContext.percentOpenLeft : overlayContext.percentOpenRight;
    } // Return the open percentage of the active swipe direction


    return overlayContext.openDirection === OpenDirection.LEFT ? overlayContext.percentOpenLeft : overlayContext.percentOpenRight;
  }, [overlayContext]);
  return { ...overlayContext,
    open,
    percentOpen
  };
}

const styles = _reactNative.StyleSheet.create({
  underlay: { ..._reactNative.StyleSheet.absoluteFillObject
  },
  flex: {
    flex: 1
  }
});
//# sourceMappingURL=index.js.map